<?php

namespace Drupal\commerce_stripe_payment_request_button\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\commerce_stripe\ErrorHelper;
use Drupal\Core\Entity\EntityInterface;
use Drupal\commerce_payment\Entity\Payment;
use Drupal\commerce_shipping\Entity\Shipment;
use Drupal\commerce_shipping\ShipmentItem;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\physical\Weight;
use Drupal\physical\WeightUnit;
use Drupal\profile\Entity\Profile;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;


class PaymentRequestButtonCheckout extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Creates a new ViewModeAccessCheck.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  public function handleOrder(EntityInterface $commerce_order, Request $request) {
    // Transform JSON values to array
    $values = Json::decode($request->getContent());
    $shipping_info = $values["data"]["result"]["shippingAddress"];
    $order = $commerce_order;

    // Init Stripe and config
    $config = \Drupal::config('commerce_payment.commerce_payment_gateway.commerce_stripe_payment_request_button')
      ->get('configuration');
    \Stripe\Stripe::setApiKey($config['secret_key']);

    // If order has problems, take it back to cart
    if (empty($order) || $order->getState()->value == 'canceled') {
      $response = [
        'redirect_uri' => Url::fromRoute(Url::fromRoute('commerce_cart.page')
          ->toString()),
      ];
      return new JsonResponse($response);
    }

    // Check if we already have payment intent
    $intent_id = $order->getData('stripe_intent');
    if ($intent_id !== NULL) {
      try {
        $intent = \Stripe\PaymentIntent::retrieve($intent_id);
        $payments = \Drupal::entityQuery('commerce_payment')
          ->condition('order_id', $order->id())
          ->execute();

        $payment = Payment::load(reset($payments));
        $payment->setRemoteId($intent->charges->data[0]->id);
        $payment->save();

        // Reset this step as it gets to NULL on cart refresh
        $order->get('checkout_flow')->setValue("payment_request_button_flow");
        $order->get('checkout_step')->setValue("payment");
        $order->save();

      } catch (\Stripe\Error\Base $e) {
        ErrorHelper::handleException($e);
      }
    }
    else {
      try {
        // Create payment intent
        $intent = \Stripe\PaymentIntent::create([
          'payment_method' => $values["data"]["payment_method_remote_id"],
          'amount' => $values["data"]["payment_request"]["total"]["amount"],
          'currency' => $values["data"]["payment_request"]["currency"],
          'payment_method_types' => ['card'],
          'capture_method' => $config['capture_method'],
          'confirmation_method' => 'automatic',
          'confirm' => TRUE,
        ]);

        if ($order->hasItems()) {
          // Put order in proper status and assign values
          $order->get('checkout_flow')->setValue("payment_request_button_flow");
          $order->get('checkout_step')->setValue("payment");
          $order->setEmail($values["data"]["result"]["payerEmail"]);
          $order->get('payment_gateway')
            ->setValue("commerce_stripe_payment_request_button");
          $order->setData('stripe_intent', $intent->id);
          $order->save();

          // Create payment for this order and give it values
          $payment_gateway = $this->entityTypeManager
            ->getStorage('commerce_payment_gateway')
            ->load('commerce_stripe_payment_request_button');

          $current_user = $order->getCustomerId();

          $payment = Payment::create([
            'state' => 'new',
            'amount' => $order->getTotalPrice(),
            'payment_gateway' => $payment_gateway->id(),
            'order_id' => $order->id(),
            'remote_id' => $intent->charges->data[0]->id,
            'payment_gateway_mode' => $payment_gateway->getPlugin()->getMode(),
            'expires' => 0,
            'uid' => $current_user,
          ]);

          if ($config['capture_method'] == "manual") {
            $payment->setState("authorization");
          }
          $payment->save();

          // Deal with billing profile
          $profile = $order->getBillingProfile();

          if (empty($profile)) {
            $profile = Profile::create([
              'type' => 'customer',
              'uid' => $current_user,
            ]);
            $profile->save();
          }

          $current_name = isset($values["data"]["result"]["payerName"]) ?: $values["data"]["result"]["paymentMethod"]["billing_details"]["name"];
          $names = explode(' ', $current_name);
          $given_name = array_shift($names);
          $family_name = implode(' ', $names);

          $profile->address->given_name = $given_name;
          $profile->address->family_name = $family_name;
          $profile->address->country_code = $values["data"]["result"]["paymentMethod"]["billing_details"]["address"]["country"];
          $profile->address->locality = $values["data"]["result"]["paymentMethod"]["billing_details"]["address"]["city"];
          $profile->address->administrative_area = $values["data"]["result"]["paymentMethod"]["billing_details"]["address"]["state"];
          $profile->address->postal_code = $values["data"]["result"]["paymentMethod"]["billing_details"]["address"]["postal_code"];
          $profile->address->address_line1 = $values["data"]["result"]["paymentMethod"]["billing_details"]["address"]["line1"];
          $profile->address->address_line2 = $values["data"]["result"]["paymentMethod"]["billing_details"]["address"]["line2"];
          if ($profile->hasField('field_phone')) {
            $profile->set('field_phone', $values["data"]["result"]["paymentMethod"]["billing_details"]["phone"]);
          }
          $profile->save();

          $order->setBillingProfile($profile);
          $order->save();

          // Deal with shipping stuff only if we selected that we want to fetch shipping address
          if (!empty($values["data"]["result"]["shippingAddress"]) && (!$order->hasField('shipments') || $order->get('shipments')
                ->isEmpty())) {
            $this->handleShipping($order, $shipping_info, $values);
          }
        }

      } catch (\Stripe\Error\Base $e) {
        // Server will return 500 to the frontend.
        $this->messenger->addError($e->getMessage());
        ErrorHelper::handleException($e);
      }
    }


    if ($intent->status == 'requires_confirmation' || $intent->status == 'requires_source_action' || $intent->status == 'requires_action') {
      # Tell the client to handle the action
      $response = [
        'requires_action' => TRUE,
        'payment_intent_client_secret' => $intent->client_secret,
        'payment_method_remote_id' => $values["data"]["payment_method_remote_id"],
        'redirect_uri' => Url::fromRoute('commerce_checkout.form', ['commerce_order' => $order->id(),])
            ->toString() . '/payment',
      ];
    }
    else {
      if ($intent->status == 'succeeded' || $intent->status == 'requires_capture') {
        # The payment didn’t need any additional actions and completed!
        # Handle post-payment fulfillment
        $response = [
          'success' => TRUE,
          'redirect_uri' => Url::fromRoute('commerce_checkout.form', ['commerce_order' => $order->id(),])
              ->toString() . '/payment',
        ];
      }
      else {
        # Invalid status
        //http_response_code(500);
        $response = ['error' => 'Invalid PaymentIntent status'];
      }
    }
    return new JsonResponse($response);
  }


  private function handleShipping($order, $shipping_info, $values) {

    $first_shipment = Shipment::create([
      'type' => 'default',
      'order_id' => $order->id(),
      'title' => 'Shipment',
      'state' => 'ready',
    ]);

    // Loop through order items and add them to shipment
    foreach ($order->getItems() as $order_item) {
      $quantity = $order_item->getQuantity();
      $purchased_entity = $order_item->getPurchasedEntity();


      //TODO Skip weight if there is none, currently breaks order flow
      if ($purchased_entity->get('weight')->isEmpty()) {
        $weight = new Weight(1, WeightUnit::GRAM);
      }
      else {
        $weight_item = $purchased_entity->get('weight')->first();
        $weight = $weight_item->toMeasurement();
      }

      $shipment_item = new ShipmentItem([
        'order_item_id' => $order_item->id(),
        'title' => $purchased_entity->label(),
        'quantity' => $quantity,
        'weight' => $weight->multiply($quantity),
        'declared_value' => $order_item->getTotalPrice(),
      ]);
      $first_shipment->addItem($shipment_item);
    }

    // Find rate, we use the first one, pretending there is only one we want to use, fixed rate for order
    $shipping_method_storage = $this->entityTypeManager->getStorage('commerce_shipping_method');
    $shipping_method = $shipping_method_storage->load($values["data"]["result"]["shippingOption"]["id"]);

    // Add proper shipping method
    $first_shipment->setShippingMethod($shipping_method);
    $shipping_method_plugin = $shipping_method->getPlugin();
    $shipping_rates = $shipping_method_plugin->calculateRates($first_shipment);
    $shipping_rate = reset($shipping_rates);
    $shipping_service = $shipping_rate->getService()->getId();
    $shipping_method_amount = $shipping_rate->getAmount();
    $first_shipment->setAmount($shipping_method_amount);

    // We only get the full name from Apple/Google pay, so we need to "guess" the given
    // name and the family name.
    $names = explode(' ', $shipping_info['recipient']);
    $given_name = array_shift($names);
    $family_name = implode(' ', $names);

    $profile = $this->entityTypeManager->getStorage('profile')->create([
      'uid' => $order->getCustomerId(),
      'type' => 'customer',
    ]);
    $profile->address->given_name = $given_name;
    $profile->address->family_name = $family_name;

    $profile->address->given_name = $given_name;
    $profile->address->family_name = $family_name;
    $profile->address->country_code = $shipping_info['country'];
    $profile->address->locality = $shipping_info['city'];
    $profile->address->administrative_area = $shipping_info['dependentLocality'];
    $profile->address->postal_code = $shipping_info['postalCode'];
    $profile->address->address_line1 = $shipping_info["addressLine"][0];
    $profile->address->address_line2 = isset($shipping_info["addressLine"][1]) ?: "";
    if ($profile->hasField('field_phone')) {
      $profile->set('field_phone', $values["data"]["result"]["payerPhone"]);
    }
    $profile->save();

    $first_shipment->setShippingProfile($profile);
    $first_shipment->setShippingService($shipping_service);
    $first_shipment->save();
    //$order->set('shipments', $first_shipment);
    //$order->save();
  }
}
